from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from django.db import models
from django.db.models.signals import post_save
from django.dispatch import receiver

from jsonfield import JSONField

from model_utils import Choices
from model_utils.models import TimeStampedModel
from model_utils.managers import InheritanceManager
#from thumbnail_works.fields import EnhancedImageField

from .utils import setup_upload_route, content_type_restriction

class ShelfManager(InheritanceManager):

    def get_query_set(self):
        return super(ShelfManager, self).get_query_set().select_subclasses().prefetch_related('audio_set', 'video_set', 'image_set')

    def by_user(self, user):
        return self.get_query_set().filter(library__user=user)


class ShelfManagerWithRelations(ShelfManager):

    def get_query_set(self):
        return super(ShelfManagerWithRelations, self).get_query_set().prefetch_related('shelfrelation_set')


class Shelf(TimeStampedModel):
    STATE_CHOICES = Choices(
            (0, 'unconverted', 'Unconverted'),
            (1, 'converted', 'Converted')
        )
    state = models.CharField(choices=STATE_CHOICES, default=STATE_CHOICES.unconverted,
                                db_index=True, max_length=2)
    name = models.CharField(max_length=255)
    library = models.ForeignKey('MediaLibrary')

    objects = ShelfManager()
    with_relations = ShelfManagerWithRelations()

    meta = JSONField(null=True, blank=True, help_text="The JSON metadata that belongs to the shelf")

    def file_set():
        doc = "The file_set property."
        def fget(self):
            raise NotImplementedError
        return locals()
    file_set = property(**file_set())

    class Meta:
        verbose_name_plural = 'Shelves'

    def __unicode__(self):
        return u'%s (%s)' % (self.name, self.__class__)

    @models.permalink
    def get_absolute_url(self):
        return ('medialibrary-shelf', (), {'pk': self.pk})

    def original():
        doc = "The original file"
        def fget(self):
            return self.file_set.filter(descriptor='original')[0]
        return locals()
    original = property(**original())

    def size():
        doc = "The size of the original file"
        def fget(self):
            return self.original.file.size
        return locals()
    size = property(**size())

    def url():
        doc = "The url of the original file"
        def fget(self):
            return self.original.file.url
        return locals()
    url = property(**url())


class ShelfRelation(TimeStampedModel):
    shelf = models.ForeignKey(Shelf, related_name='relationships')
    content_type = models.ForeignKey(ContentType, limit_choices_to=content_type_restriction)
    object_id = models.PositiveIntegerField()
    relates_to = generic.GenericForeignKey('content_type', 'object_id')


class AudioShelf(Shelf):

    # AVAILABLE_FORMATS = ('aac', 'ogg', 'webm')
    ALLOWED_FORMATS = ('mp3', 'aac', 'ogg', 'webm')

    def file_set():
        doc = "The file_set property."
        def fget(self):
            return self.audio_set
        return locals()
    file_set = property(**file_set())


class VideoShelf(Shelf):

    # AVAILABLE_FORMATS = ('mp4', 'webm')
    ALLOWED_FORMATS = (
    '3gp', 'asf', 'avi', 'divx', 'flv', 'mkv', 'mov', 'mpg', 'mp4', 'mpeg',
    'm4v', 'mxf', 'ogg', 'vob', 'webm')
    thumbnails = models.ManyToManyField('ImageShelf', through='VideoThumbnail', null=True, blank=True)

    def file_set():
        doc = "The file_set property."
        def fget(self):
            return self.video_set
        return locals()
    file_set = property(**file_set())

    def thumbnail():
        doc = "The video's thumbnail"
        def fget(self):
            try:
                return VideoThumbnail.objects.get_selected(video=self).image
            except VideoThumbnail.DoesNotExist:
                return None
        def fset(self, imageShelf):
            try:
                thumbnail = VideoThumbnail.objects.get(video=self, image=imageShelf)
            except VideoThumbnail.DoesNotExist:
                VideoThumbnail.objects.create(video=self, image=imageShelf, selected=True)
            else:
                thumbnail.selected = True
                thumbnail.save()
        return locals()
    thumbnail = property(**thumbnail())

    def duration():
        doc = "The length property."
        def fget(self):
            return self.original.duration
        return locals()
    duration = property(**duration())


class ImageShelf(Shelf):

    ALLOWED_FORMATS = ('jpg', 'jpeg', 'gif', 'png', 'pdf')

    def file_set():
        doc = "The file_set property."
        def fget(self):
            return self.image_set
        return locals()
    file_set = property(**file_set())


class ThumbnailManager(models.Manager):
    use_for_related_fields = True

    def get_selected(self, **kwargs):
        kwargs.update({'selected': True})
        return self.get_query_set().get(**kwargs)


class VideoThumbnail(models.Model):

    video = models.ForeignKey(VideoShelf)
    image = models.ForeignKey(ImageShelf)
    selected = models.BooleanField(default=False)

    objects = ThumbnailManager()

    def save(self, **kwargs):
        if self.selected == True:
            VideoThumbnail.objects.filter(video=self.video).update(selected=False)
        return super(VideoThumbnail, self).save(**kwargs)

class BaseFile(TimeStampedModel):
    """
        BaseFile abstract class that defines all the common characters.
    """
    shelf = models.ForeignKey(Shelf)
    descriptor = models.CharField(max_length=255, blank=True, default='original')
    content_type = models.CharField(max_length=50, blank=True, default='application/octet-stream')
    file = models.FileField(max_length=255, upload_to=setup_upload_route)
    meta = JSONField(blank=True, help_text="An arbitrary JSON")

    class Meta:
        abstract = True

    def has_valid_format(self):
        return self.file.url.split('.')[-1].lower() in self.shelf.ALLOWED_FORMATS

    def save(self, force_insert=False, force_update=False, update_fields=None):
        if not self.has_valid_format():
            raise ValueError
        return super(BaseFile, self).save(force_insert, force_update, update_fields)

    def save_alternative(self, url, descriptor, meta={}, **kwargs):
        """Creates a new media record in the DB from the current one without saving any new files in the storage"""
        self.pk = None
        self.file.name = url
        self.descriptor = descriptor
        self.meta = meta if meta else self.meta
        for key, value in kwargs.items():
            setattr(self, key, value)
        self.save()
        return self


class Audio(BaseFile):
    """
        Audio model to store audiofiles that can be added to a project.
    """
    TYPES = Choices('original', 'webm', 'mp3')


class Video(BaseFile):
    """
    Audio model to store audiofiles that can be added to a project.
    """
    TYPES = Choices('original', 'webm', 'mp4')

    duration = models.IntegerField(null=True)
    height = models.IntegerField(null=True)
    width = models.IntegerField(null=True)


class Image(BaseFile):
    TYPES = Choices('original', 'thumbnail')


class MediaLibrary(TimeStampedModel):
    """
        Media Library for a user, it aggregates the three other sublibraries,
        so the user can easily reach them.
    """

    user = models.OneToOneField(User)

    def __unicode__(self):
        return 'Library for user %s' % self.user.username

@receiver(post_save, sender=User, dispatch_uid="create_media_library")
def create_media_library(sender, created, instance, **kwargs):
    if created:
        instance.medialibrary, created = MediaLibrary.objects.get_or_create(user=instance)
        instance.save()



import datetime

def setup_upload_route(instance, filename=None):
    today = datetime.datetime.today()
    return '%s/%s-%02d-%02d/%s' % (instance.shelf.__class__.__name__.lower(),
                                       today.year, today.month, today.day,
                                       filename)

content_type_restriction = None


from django.contrib.auth.models import User
from django.core.files import File
from django.test import TestCase
from django.contrib.contenttypes.models import ContentType
from . import TearDownMixin
from ..models import MediaLibrary, AudioShelf, Shelf, Audio, \
    VideoShelf, VideoThumbnail, ImageShelf, Video

class LibraryTest(TestCase):

    def test_library_access(self):
        """Test that library types can be properly accessed"""
        user = User.objects.create(username='testuser')
        self.assertIsInstance(user.medialibrary, MediaLibrary)
        

class ShelfTest(TearDownMixin, TestCase):

    def setUp(self):
        self.user = User.objects.create(username='testuser')

    def test_shelf_type(self):
        """The proper shelf is always returned"""
        audio = AudioShelf.objects.create(name='testaudio', library=self.user.medialibrary)
        shelves = Shelf.objects.all()
        self.assertIsInstance(shelves[0], AudioShelf)

    def test_file_format_check_on_save(self):
        """The uploaded file should have the proper format, save should be denied otherwise"""
        shelf = AudioShelf.objects.create(name='testaudio', library=self.user.medialibrary)

        media = Audio(file=File(open(__file__, 'rb'), 'testaudio.mp3'))
        shelf.audio_set.add(media)
        self.assertTrue(media.has_valid_format())
        
        media = Audio(file=File(open(__file__, 'rb'), 'testaudio.xxx'))
        self.assertRaises(ValueError, shelf.audio_set.add, media)

    def test_fetching_files_works(self):
        shelf = AudioShelf.objects.create(name='testaudio', library=self.user.medialibrary)
        media = Audio(file=File(open(__file__, 'rb'), 'testaudio.mp3'))
        shelf.audio_set.add(media)

        shelf = AudioShelf.objects.get(pk=shelf.pk)
        self.assertEqual(shelf.audio_set.all()[0], media)

    def test_original_file(self):
        shelf1 = AudioShelf.objects.create(name='testaudio', library=self.user.medialibrary)
        media1 = Audio(file=File(open(__file__, 'rb'), 'testaudio.mp3'), descriptor='original')
        shelf1.audio_set.add(media1)

        shelf2 = VideoShelf.objects.create(name='testVideo', library=self.user.medialibrary)
        media2 = Video(file=File(open(__file__, 'rb'), 'testVideo.mp4'), descriptor='original')
        shelf2.video_set.add(media2)

        self.assertEqual(shelf1.original, media1)
        self.assertEqual(shelf2.original, media2)


class ShelfWithRelationshipTest(TestCase):

    def setUp(self):
        self.user = User.objects.create(username='testuser')

    def test_relationship_works(self):
        shelf = AudioShelf.objects.create(name='testaudio', library=self.user.medialibrary)
        shelf.relationships.create(relates_to=self.user)
        ct_user = ContentType.objects.get_for_model(self.user)
        shelfrelations = AudioShelf.objects.get(relationships__content_type__pk=ct_user.pk, relationships__object_id=self.user.pk)
        self.assertEqual(shelfrelations, shelf)

    def test_relationshipmanager(self):
        shelf = AudioShelf.objects.create(name='testaudio', library=self.user.medialibrary)
        with self.settings(DEBUG=True):
            shelf.relationships.create(relates_to=self.user)
            from django.db import connection
            q1 = len(connection.queries)
            AudioShelf.with_relations.filter(pk=shelf.pk)
            q2 = len(connection.queries)
            self.assertEqual(q2-q1, 0)
        

class BaseFileTest(TearDownMixin, TestCase):

    def setUp(self):
        self.user = User.objects.create(username='testuser')

    def test_save_alternative(self):
        shelf = AudioShelf.objects.create(name='testaudio', library=self.user.medialibrary)
        media = Audio(file=File(open(__file__, 'rb'), 'testaudio.mp3'))
        shelf.audio_set.add(media)

        newmedia = media.save_alternative('me/testaudio2.mp3', 'mp3')
        self.assertEqual(shelf.audio_set.count(), 2)
        

class VideoShelfTest(TearDownMixin, TestCase):

    def setUp(self):
        self.user = User.objects.create(username='testuser')

    def test_thumbnail_set(self):
        shelf = VideoShelf.objects.create(name='testvideo', library=self.user.medialibrary)
        image = ImageShelf.objects.create(name='testimage', library=self.user.medialibrary)
        shelf.thumbnail = image
        self.assertEqual(shelf.thumbnail, image)

    def test_change_thumbnail(self):
        self.test_only_one_thumbnail()
        shelf.thumbnail = image1
        self.assertEqual(shelf.thumbnail, image1)

    def test_thumbnail_get(self):
        shelf = VideoShelf.objects.create(name='testvideo', library=self.user.medialibrary)
        image = ImageShelf.objects.create(name='testimage', library=self.user.medialibrary)
        VideoThumbnail.objects.create(video=shelf, image=image, selected=True)
        self.assertEqual(shelf.thumbnail, image)

    def test_only_one_thumbnail(self):
        shelf = VideoShelf.objects.create(name='testvideo', library=self.user.medialibrary)
        image1 = ImageShelf.objects.create(name='testimage1', library=self.user.medialibrary)
        image2 = ImageShelf.objects.create(name='testimage2', library=self.user.medialibrary)
        VideoThumbnail.objects.create(video=shelf, image=image1, selected=True)
        VideoThumbnail.objects.create(video=shelf, image=image2, selected=True)
        self.assertEqual(shelf.thumbnail, image2)
        return shelf, image1

    def test_change_thumbnail(self):
        shelf, image1 = self.test_only_one_thumbnail()
        shelf.thumbnail = image1
        self.assertEqual(VideoShelf.objects.get(pk=shelf.pk).thumbnail, image1)



from rest_framework import serializers
from .models import Audio, Image, Video, AudioShelf, ImageShelf, VideoShelf

def basefileserializer_factory(mymodel):

    class ThisClass(serializers.ModelSerializer):
        # imagefile = serializers.ImageField()
        # thumbnail = serializers.SerializerMethodField('get_thumbnail_url')
        url = serializers.SerializerMethodField('get_file_url')

        class Meta:
            model = mymodel
            fields = ('file', 'descriptor', 'meta', 'url')

        def get_thumbnail_url(self, obj):
            return obj.imagefile.thumb.url

        def get_file_url(self, obj):
            return obj.file.url

    return ThisClass

ImageSerializer = basefileserializer_factory( Image )
AudioSerializer = basefileserializer_factory( Audio )
VideoSerializer = basefileserializer_factory( Video )

basefileserializer_registry = {
    'image': ImageSerializer,
    'audio': AudioSerializer,
    'video': VideoSerializer,
}

class ShelfSerializer(serializers.ModelSerializer):

    library = serializers.PrimaryKeyRelatedField(required=True, many=False)
    url = serializers.Field(source='get_absolute_url')

    def __init__(self, *args, **kwargs):
        self.shelf_type = kwargs['context'].get('shelf_type', None)
        method = kwargs['context'].get('method', None)

        if not self.shelf_type:
            raise ValueError('Shelf type must be specified in the ShelfSerializer context')

        if kwargs.get('files', None) and len( kwargs['files'].keys() ) > 1:
            raise ValueError('Only a single file can be uploaded at a time')

        shelf_types = {
            'image': ImageShelf,
            'audio': AudioShelf,
            'video': VideoShelf,
        }

        self.Meta.model = shelf_types[self.shelf_type]

        super(ShelfSerializer, self).__init__(*args, **kwargs)

        if method == 'POST':  #: we are creating a shelve
            self.file_serializer = basefileserializer_registry[self.shelf_type](
                data={}, files=kwargs.pop('files')
            )
        elif method == 'PUT':  #: we are updating a shelve
            files = serializers.PrimaryKeyRelatedField(
                source='%s_set.add' % self.shelf_type,
                required=True, many=False
            )
        else:
            self.fields['%s_set' % self.shelf_type] = basefileserializer_registry[self.shelf_type](many=True)
            self.fields['files'] = serializers.SerializerMethodField('get_files')

    def validate(self, data):
        if not self.file_serializer.is_valid():
            raise serializers.ValidationError("The attached file did not validate: %s" % str(self.file_serializer.errors))
        return data

    def save_file(self, shelf):
        self.file_serializer.object.shelf = shelf
        newfile = self.file_serializer.save()
        return newfile

    def save(self, **kwargs):
        shelf = super(ShelfSerializer, self).save()
        self.save_file(shelf)
        return shelf

    def get_files(self, obj):
        return getattr(self, '%s_set' % self.shelf_type, [])

    def to_native(self, obj):
        """As get_files usually evaluates to [], we add its value here as well"""
        data = super(ShelfSerializer, self).to_native(obj)
        if data.has_key('files') and not data['files']:
            data['files'] = data['%s_set' % self.shelf_type]
        return data

    class Meta:
        fields = ('url', 'name', 'state', 'library', 'meta')
        read_only_fields = ('state', )




import shutil


class TearDownMixin(object):
    """
    This mixin removes all the test files (and folders) created by the tests
    """

    @classmethod
    def tearDownClass(cls):
        try:
            shutil.rmtree('test_temp')
        except OSError:
            pass


from models import *
from serializers import *
from views import *


from django.test import TestCase
from django.contrib.auth.models import User
from django.core.files import File
from . import TearDownMixin
from ..serializers import ShelfSerializer, AudioSerializer
from ..models import ImageShelf, Image, AudioShelf, Audio


class BaseFileSerializerTest(TestCase):

    def setUp(self):
        self.user = User.objects.create_user('alma', 'alma@example.com', 'alma')
    
    def test_audio_serializer_return_values(self):
        shelf = AudioShelf.objects.create(name='testaudio', library=self.user.medialibrary)
        audio = Audio(file=File(open(__file__, 'rb'), 'testaudio.mp3'))
        # shelf.audio_set.add(media)

        serializer = AudioSerializer(audio)
        self.assertEqual(serializer.data.keys(), ['file', 'descriptor', 'meta', 'url'])
        self.assertEqual(serializer.data['url'], 'testaudio.mp3')


class ShelfSerializerTest(TearDownMixin, TestCase):

    def setUp(self):
        self.user = User.objects.create_user('alma', 'alma@example.com', 'alma')

    def test_create_shelf_with_file(self):
        data = {
            'name': 'Example',
            'library': self.user.medialibrary.pk,
        }
        files = {'file': File(open(__file__, 'rb'), 'testimage.png')}
        context = {
            'shelf_type': 'image',
            'method': 'POST',
        }
        serialized = ShelfSerializer(data=data, files=files, context=context)
        self.assertTrue(serialized.is_valid(), str(serialized.errors))
        shelf = serialized.save()
        
        s1 = ImageShelf.objects.all()[0]
        self.assertEqual(shelf, s1)
        self.assertEqual(shelf.image_set.all()[0], Image.objects.all()[0])

    def test_get_shelf_with_files(self):
        shelf = AudioShelf.objects.create(name='testaudio', library=self.user.medialibrary)
        media = Audio(file=File(open(__file__, 'rb'), 'testaudio.mp3'))
        shelf.audio_set.add(media)

        context = {
            'shelf_type': 'audio',
        }
        serializer = ShelfSerializer(shelf, context=context)
        self.assertEqual(len(serializer.data['audio_set']), 1)
        self.assertEqual(len(serializer.data['files']), 1)

        fileSerializer = AudioSerializer(media)
        self.assertEqual(serializer.data['audio_set'], [fileSerializer.data])
        self.assertEqual(serializer.data['files'], [fileSerializer.data])



from django.contrib import admin
from .models import Shelf, Audio, Video, Image

class AudioInline(admin.TabularInline):
    model = Audio
    extra = 0

class ImageInline(admin.TabularInline):
    model = Image
    extra = 0

class VideoInline(admin.TabularInline):
    model = Video
    extra = 0

class ShelfAdmin(admin.ModelAdmin):
    list_display =('pk', 'name', 'state', 'library_user', 'modified', 'created')
    date_hierarchy = 'created'
    list_filter  = ('state',)
    search_fields = ('name', )
    link_fields = ('pk', 'name')
    inlines = [
        AudioInline, ImageInline, VideoInline
    ]

    def library_user(self, obj):
        return obj.library.user
    library_user.allow_tags = False
    library_user.short_description = 'Library for'

admin.site.register(Shelf, ShelfAdmin)



from django.conf.urls import patterns, url
from .views import MediaLibraryAPIView, MediaLibraryItemView, AddShelfRelationAPIView

urlpatterns = patterns('',
    url(r'^(?P<type>(audio|video|image))/$', MediaLibraryAPIView.as_view(), name='medialibrary'),
    url(r'^(?P<pk>\d+)/$', MediaLibraryItemView.as_view(), name='medialibrary-shelf'),
    url(r'^(?P<pk>\d+)/add/$', AddShelfRelationAPIView.as_view(), name='medialibrary-shelf-add-relation')
)



import json
from django.test import TestCase
from django.core.files import File
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse

from . import TearDownMixin
from ..models import AudioShelf, Audio, Shelf
from ..serializers import ShelfSerializer

class MyViewTestMixin(object):

    def _create_user(self, username='alma'):
        self.user = User.objects.create_user(username, '%s@example.com' % username, username)

    def _login(self):
        login = self.client.login(username='alma', password='alma')
        self.assertTrue(login)


class AddShelfRelationAPIView(MyViewTestMixin, TestCase):

    def setUp(self):
        self._create_user()
        self.shelf = AudioShelf.objects.create(name='testaudio', library=self.user.medialibrary)

    def test_requires_login(self):
        resp = self.client.post(reverse('medialibrary-shelf-add-relation', kwargs={'pk':self.shelf.pk}),
            data={})
        self.assertEqual(resp.status_code, 403)

    def test_add_relationship(self):
        self._login()
        resp = self.client.post(reverse('medialibrary-shelf-add-relation', kwargs={'pk':self.shelf.pk}),
            data={'model':'auth.user', 'object_id':self.user.pk})
        self.assertEqual(resp.status_code, 201, resp.content)
        self.assertEqual(self.shelf.relationships.count(), 1)

    def test_relationships_are_restricted(self):
        self._login()
        resp = self.client.post(reverse('medialibrary-shelf-add-relation', kwargs={'pk':self.shelf.pk}),
            data={'model':'medialibrary.medialibrary', 'object_id':self.user.medialibrary.pk})
        self.assertEqual(resp.status_code, 400, resp.content)
        data = json.loads(resp.content)
        self.assertTrue(data.has_key('model'))


class LibraryViewTest(TearDownMixin, MyViewTestMixin, TestCase):

    def setUp(self):
        self._create_user()

    def test_upload_image(self):
        self._login()
        with open('resources/test.png') as fp:
            resp = self.client.post(reverse('medialibrary',
                                            kwargs={'type':'image'}),
                                    {'file': fp})
        self.assertEqual(resp.status_code, 201)
        data = json.loads(resp.content)
        self.assertEqual(data['name'], 'test.png')
        self.assertEqual(len(data['files']), 1)

    def test_upload_set_meta(self):
        self._login()
        with open('resources/test.png') as fp:
            resp = self.client.post(reverse('medialibrary',
                                            kwargs={'type':'image'}),
                                    {'file': fp, 'meta': json.dumps({'yolo': 'swag'})})

        self.assertEqual(resp.status_code, 201)
        data = json.loads(resp.content)
        self.assertEqual(data['name'], 'test.png')
        shelf = Shelf.objects.get(pk=1)

        self.assertTrue(data.has_key('meta'))
        self.assertEqual(shelf.meta, {'yolo': 'swag'})


class LibraryLoadedViewTest(TearDownMixin, MyViewTestMixin, TestCase):

    # fixtures = ['test_library.json']

    def setUp(self):
        self._create_user()
        self.shelf = AudioShelf.objects.create(name='testaudio', library=self.user.medialibrary)
        media1 = Audio(file=File(open(__file__, 'rb'), 'testaudio.mp3'))
        media2 = Audio(file=File(open(__file__, 'rb'), 'testaudio.mp3'))
        self.shelf.audio_set.add(media1)
        self.shelf.audio_set.add(media2)

    def test_get_audio(self):
        self._login()
        resp = self.client.get(reverse('medialibrary', kwargs={'type':'audio'}))
        self.assertEqual(resp.status_code, 200)
        keys_to_match = json.loads(resp.content)[0].keys()
        keys_to_match.sort()
        serializer_keys = ShelfSerializer(self.shelf, context={'shelf_type': 'audio'}).data.keys()
        serializer_keys.sort()
        self.assertEqual(
            keys_to_match,
            serializer_keys
        )

    def test_returned_items(self):
        """VIDZOR-311"""
        self._login()
        resp = self.client.get(reverse('medialibrary', kwargs={'type':'video'}))
        data = json.loads(resp.content)
        self.assertEqual(data, [])


class LibraryPermissionTest(TearDownMixin, MyViewTestMixin, TestCase):

    def setUp(self):
        self._create_user()
        self.username2 = 'alma2'
        self.user2 = User.objects.create_user(self.username2, '%s@example.com' % self.username2, self.username2)

        # upload a testaudio file for user2
        self.shelf = AudioShelf.objects.create(name='testaudio', library=self.user2.medialibrary)
        media = Audio(file=File(open(__file__, 'rb'), 'testaudio.mp3'))
        self.shelf.audio_set.add(media)

    def test_audioshelf_permission(self):
        # login with alma2
        login = self.client.login(username=self.username2, password=self.username2)
        self.assertTrue(login)

        # get audio with alma2
        resp = self.client.get(reverse('medialibrary', kwargs={'type':'audio'}))
        self.assertContains(resp, 'testaudio')

        # logout and login with other user
        self.client.logout()
        self._login()

        # other user should not see the testaudio file
        resp = self.client.get(reverse('medialibrary', kwargs={'type':'audio'}))
        self.assertNotContains(resp, 'testaudio')


from django.contrib.contenttypes.models import ContentType

from rest_framework import generics, permissions
from .models import Shelf, ShelfRelation
from .serializers import ShelfSerializer


class AddShelfRelationAPIView(generics.CreateAPIView):
    model = ShelfRelation
    # TODO: should be allowed only for the owner
    permission_classes = (permissions.IsAuthenticated,) 
    
    def post(self, request, *args, **kwargs):
        request.DATA['shelf'] = kwargs['pk']
        request.DATA['content_type'] = ContentType.objects.get_by_natural_key(request.DATA['model'].split('.')[0], request.DATA['model'].split('.')[1]).pk
        resp = super(AddShelfRelationAPIView, self).post(request, *args, **kwargs)
        if resp.status_code == 400 and resp.data.has_key('content_type'):
            resp.data = {'model': "Invalid model given"}
        return resp
        

class MediaLibraryItemView(generics.RetrieveUpdateAPIView):
    model = Shelf
    # TODO: should be allowed only for the owner
    permission_classes = (permissions.IsAuthenticated,) 


class MediaLibraryAPIView(generics.ListCreateAPIView):
    """Retrieves and creates shelves of a given type from the MediaLibrary"""
    model = Shelf
    serializer_class = ShelfSerializer
    permission_classes = (permissions.IsAuthenticated,)

    def get(self, request, *args, **kwargs):
        self.shelf_type = kwargs.get('type')
        return super(MediaLibraryAPIView, self).get(request, *args, **kwargs)

    def get_queryset(self):
        qs = self.model.objects.by_user(self.request.user)
        return qs.exclude(**{'%sshelf__isnull' % self.shelf_type:True})

    def get_serializer_context(self):
        ctx = super(MediaLibraryAPIView, self).get_serializer_context()
        ctx['shelf_type'] = self.shelf_type
        ctx['method'] = self.request.method
        return ctx

    def post(self, request, *args, **kwargs):
        self.shelf_type = kwargs.get('type')
        request.DATA['library'] = self.request.user.medialibrary.pk
        request.DATA['name'] = request.FILES.values()[0].name

        resp = super(MediaLibraryAPIView, self).post(request, *args, **kwargs)
        if not resp.data.has_key('url'):
            return resp
            
        new_id = resp.data['url'].split('/')[-2]
        new_shelve = self.get_queryset().get(pk=new_id)
        context = self.get_serializer_context()
        context['method'] = 'GET'
        resp_serializer = self.serializer_class(new_shelve, context=context)
        resp.data = resp_serializer.data
        return resp

    def pre_save(self, obj):
        obj.library = self.request.user.medialibrary
        super(MediaLibraryAPIView, self).pre_save(obj)




# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'Video.duration'
        db.add_column('medialibrary_video', 'duration',
                      self.gf('django.db.models.fields.IntegerField')(null=True),
                      keep_default=False)

        # Adding field 'Video.height'
        db.add_column('medialibrary_video', 'height',
                      self.gf('django.db.models.fields.IntegerField')(null=True),
                      keep_default=False)

        # Adding field 'Video.width'
        db.add_column('medialibrary_video', 'width',
                      self.gf('django.db.models.fields.IntegerField')(null=True),
                      keep_default=False)


    def backwards(self, orm):
        # Deleting field 'Video.duration'
        db.delete_column('medialibrary_video', 'duration')

        # Deleting field 'Video.height'
        db.delete_column('medialibrary_video', 'height')

        # Deleting field 'Video.width'
        db.delete_column('medialibrary_video', 'width')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'medialibrary.audio': {
            'Meta': {'object_name': 'Audio'},
            'created': ('model_utils.fields.AutoCreatedField', [], {'default': 'datetime.datetime.now'}),
            'descriptor': ('django.db.models.fields.CharField', [], {'default': "'original'", 'max_length': '255', 'blank': 'True'}),
            'file': ('django.db.models.fields.files.FileField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'meta': ('jsonfield.fields.JSONField', [], {'blank': 'True'}),
            'modified': ('model_utils.fields.AutoLastModifiedField', [], {'default': 'datetime.datetime.now'}),
            'shelf': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['medialibrary.Shelf']"})
        },
        'medialibrary.audioshelf': {
            'Meta': {'object_name': 'AudioShelf', '_ormbases': ['medialibrary.Shelf']},
            'shelf_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['medialibrary.Shelf']", 'unique': 'True', 'primary_key': 'True'})
        },
        'medialibrary.image': {
            'Meta': {'object_name': 'Image'},
            'created': ('model_utils.fields.AutoCreatedField', [], {'default': 'datetime.datetime.now'}),
            'descriptor': ('django.db.models.fields.CharField', [], {'default': "'original'", 'max_length': '255', 'blank': 'True'}),
            'file': ('django.db.models.fields.files.FileField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'meta': ('jsonfield.fields.JSONField', [], {'blank': 'True'}),
            'modified': ('model_utils.fields.AutoLastModifiedField', [], {'default': 'datetime.datetime.now'}),
            'shelf': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['medialibrary.Shelf']"})
        },
        'medialibrary.imageshelf': {
            'Meta': {'object_name': 'ImageShelf', '_ormbases': ['medialibrary.Shelf']},
            'shelf_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['medialibrary.Shelf']", 'unique': 'True', 'primary_key': 'True'})
        },
        'medialibrary.medialibrary': {
            'Meta': {'object_name': 'MediaLibrary'},
            'created': ('model_utils.fields.AutoCreatedField', [], {'default': 'datetime.datetime.now'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('model_utils.fields.AutoLastModifiedField', [], {'default': 'datetime.datetime.now'}),
            'user': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['auth.User']", 'unique': 'True'})
        },
        'medialibrary.shelf': {
            'Meta': {'object_name': 'Shelf'},
            'created': ('model_utils.fields.AutoCreatedField', [], {'default': 'datetime.datetime.now'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'library': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['medialibrary.MediaLibrary']"}),
            'modified': ('model_utils.fields.AutoLastModifiedField', [], {'default': 'datetime.datetime.now'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'state': ('django.db.models.fields.CharField', [], {'default': '0', 'max_length': '2', 'db_index': 'True'})
        },
        'medialibrary.shelfrelation': {
            'Meta': {'object_name': 'ShelfRelation'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'created': ('model_utils.fields.AutoCreatedField', [], {'default': 'datetime.datetime.now'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('model_utils.fields.AutoLastModifiedField', [], {'default': 'datetime.datetime.now'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'shelf': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'relationships'", 'to': "orm['medialibrary.Shelf']"})
        },
        'medialibrary.video': {
            'Meta': {'object_name': 'Video'},
            'created': ('model_utils.fields.AutoCreatedField', [], {'default': 'datetime.datetime.now'}),
            'descriptor': ('django.db.models.fields.CharField', [], {'default': "'original'", 'max_length': '255', 'blank': 'True'}),
            'duration': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'file': ('django.db.models.fields.files.FileField', [], {'max_length': '100'}),
            'height': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'meta': ('jsonfield.fields.JSONField', [], {'blank': 'True'}),
            'modified': ('model_utils.fields.AutoLastModifiedField', [], {'default': 'datetime.datetime.now'}),
            'shelf': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['medialibrary.Shelf']"}),
            'width': ('django.db.models.fields.IntegerField', [], {'null': 'True'})
        },
        'medialibrary.videoshelf': {
            'Meta': {'object_name': 'VideoShelf', '_ormbases': ['medialibrary.Shelf']},
            'shelf_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['medialibrary.Shelf']", 'unique': 'True', 'primary_key': 'True'}),
            'thumbnails': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'to': "orm['medialibrary.ImageShelf']", 'null': 'True', 'through': "orm['medialibrary.VideoThumbnail']", 'blank': 'True'})
        },
        'medialibrary.videothumbnail': {
            'Meta': {'object_name': 'VideoThumbnail'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['medialibrary.ImageShelf']"}),
            'selected': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'video': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['medialibrary.VideoShelf']"})
        }
    }

    complete_apps = ['medialibrary']


import os
from setuptools import setup, find_packages


def read_file(filename):
    """Read a file into a string"""
    path = os.path.abspath(os.path.dirname(__file__))
    filepath = os.path.join(path, filename)
    try:
        return open(filepath).read()
    except IOError:
        return ''


setup(
    name='django-medialibrary',
    version=__import__('medialibrary').__version__,
    author='Viktor Nagy',
    author_email='v@pulilab.com',
    packages=find_packages(),
    include_package_data=True,
    url='https://github.com/pulilab/django-medialibrary',
    license='BSD 3-Clause License, see the LICENSE file for details',
    description=u' '.join(__import__('medialibrary').__doc__.splitlines()).strip(),
    classifiers=[
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Intended Audience :: Developers',
        'Programming Language :: Python',      
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Framework :: Django',
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
    ],
    long_description=read_file('README.rst'),
    test_suite="runtests.runtests",
    zip_safe=False,
    install_requires=[
        'Django>=1.4,<1.6',
        'django-model-utils',
        'django-jsonfield>=0.9.10',
        'djangorestframework>=2.3.12'
    ],
    download_url='https://github.com/pulilab/django-medialibrary/archive/1.1.0.zip'
)
